package org.gersteinlab.regulation.numeric;

/**
 *	<pre>
 *	A class that represents a differentiable univariate real function. Each
 *	inherited class should implement at least one of df(double) and df().
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (August 29, 2008)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */
public abstract class DifferentiableFunction extends Function
{
	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	/**
	 *	Evaluate the first derivative of the function at a point.
	 *	@param		x			The point
	 *	@return					The first derivative of the function at the point
	 */
	public double df(double x)
	{
		set(x);
		return df();
	}

	/**
	 *	Evaluate the first derivative of the function at the stored
	 *	point.
	 *	@return					The first derivative of the function at the point
	 */
	public double df()
	{
		return df(xStored);
	}
}