package org.gersteinlab.regulation.numeric;

/**
 *	<pre>
 *	A class that represents a differentiable multivariate real function.
 *	Each inherited class should implement at least one of gf(double) and
 *	gf().
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (August 29, 2008)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */
public abstract class DifferentiableMultivariateFunction extends MultivariateFunction
{
	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	/**
	 *	Evaluate the gradient of the function at a point.
	 *	@param		x			The point
	 *	@return					The gradient of the function at the point
	 */
	public double[] gf(double[] x)
	{
		set(x);
		return gf();
	}

	/**
	 *	Evaluate the gradient of the function at the stored point.
	 *	@return					The gradient of the function at the point
	 */
	public double[] gf()
	{
		return gf(xStored);
	}
}