package org.gersteinlab.regulation.numeric;

/**
 *	<pre>
 *	A class that represents a doubly differentiable univariate real
 *	function. Each inherited class should implement at least one of
 *	d2f(double) and d2f().
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (August 29, 2008)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */
public abstract class DoublyDifferentiableFunction extends DifferentiableFunction
{
	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	/**
	 *	Evaluate the second derivative of the function at a point.
	 *	@param		x			The point
	 *	@return					The second derivative of the function at the point
	 */
	public double d2f(double x)
	{
		set(x);
		return d2f();
	}

	/**
	 *	Evaluate the second derivative of the function at the stored point.
	 *	@return					The second derivative of the function at the point
	 */
	public double d2f()
	{
		return d2f(xStored);
	}
}