package org.gersteinlab.regulation.numeric;

/**
 *	<pre>
 *	A class that represents a doubly differentiable multivariate real
 *	function. Each inherited class should implement at least one of
 *	hf(double) and hf().
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (August 29, 2008)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */
public abstract class DoublyDifferentiableMultivariateFunction extends DifferentiableMultivariateFunction
{
	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	/**
	 *	Evaluate the Hessian of the function at a point.
	 *	@param		x			The point
	 *	@return					The Hessian of the function at the point
	 */
	public double[][] hf(double[] x)
	{
		set(x);
		return hf();
	}

	/**
	 *	Evaluate the Hessian of the function at the stored point.
	 *	@return					The Hessian of the function at the point
	 */
	public double[][] hf()
	{
		return hf(xStored);
	}
}