package org.gersteinlab.regulation.numeric;

/**
 *	<pre>
 *	A class that represents a univariate real function. Each inherited class
 *	should implement at least one of f(double) and f().
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (August 29, 2008)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */
public abstract class Function
{
	/*----------------------------------------------------------------------
	 Instance variables
	----------------------------------------------------------------------*/

	protected double xStored = Double.NaN;		// The stored point


	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	/**
	 *	Evaluate the value of the function at a point.
	 *	@param		x			The point
	 *	@return					The value of the function at the point
	 */
	public double f(double x)
	{
		set(x);
		return f();
	}

	/**
	 *	Store the next point to be evaluated.
	 *	@param		x			The point
	 */
	public void set(double x)
	{
		this.xStored = x;
	}

	/**
	 *	Evaluate the value of the function at the last stored point.
	 *	@return					The value of the function at the point
	 */
	public double f()
	{
		return f(xStored);
	}
}