package org.gersteinlab.regulation.util;

import java.util.*;

/**
 *	<pre>
 *	This class represents a modified version of StringTokenizer that returns
 *	an empty string between two consecutive delimiters.
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (October 22, 2007)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */
public class StringTokenizer2 implements Enumeration
{
	/*----------------------------------------------------------------------
	 Instance variables
	----------------------------------------------------------------------*/

	/**
	 *	The underlying string tokenizer
	 */
	protected StringTokenizer st = null;
	/**
	 *	The delimiters
	 */
	protected String delim = null;
	/**
	 *	Whether to return delimiters
	 */
	protected boolean returnDelims = false;
	/**
	 *	Data buffer
	 */
	protected LinkedList<String> buffer = null;
	/**
	 *	Whether the last element read from st is a delimiter
	 */
	protected boolean lastIsDelim = true;


	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	public StringTokenizer2(String str)
	{
		this.delim = " \t\n\r\f";
		this.returnDelims = false;
		st = new StringTokenizer(str, this.delim, true);
		buffer = new LinkedList<String>();
	}

	public StringTokenizer2(String str, String delim)
	{
		this.delim = delim;
		this.returnDelims = false;
		st = new StringTokenizer(str, delim, true);
		buffer = new LinkedList<String>();
	}

	public StringTokenizer2(String str, String delim, boolean returnDelims)
	{
		this.delim = delim;
		this.returnDelims = returnDelims;
		st = new StringTokenizer(str, delim, true);
		buffer = new LinkedList<String>();
	}

	public int countTokens()
	{
		boolean localLastIsDelim = lastIsDelim;
		while (st.hasMoreTokens())
		{
			String token = st.nextToken();
			if (localLastIsDelim == true)
			{
				if (delim.indexOf(token) != -1)
				{
					buffer.addLast("");
					if (returnDelims)
						buffer.addLast(token);
				}
				else
				{
					localLastIsDelim = false;
					buffer.addLast(token);
				}
			}
			else
			{
				if (delim.indexOf(token) != -1)
				{
					localLastIsDelim = true;
					if (returnDelims)
						buffer.addLast(token);
				}
				else
					buffer.addLast(token);
			}
		}
		if (localLastIsDelim)
			buffer.addLast("");
		return buffer.size();
	}

	public boolean hasMoreElements()
	{
		return buffer.size() != 0 || st.hasMoreElements();
	}

	public boolean hasMoreTokens()
	{
		return hasMoreElements();
	}

	public Object nextElement() throws NoSuchElementException
	{
		if (buffer.size() != 0)
		{
			Object obj = buffer.getFirst();
			buffer.removeFirst();
			return obj;
		}
		else if (!st.hasMoreElements() && lastIsDelim)
		{
			lastIsDelim = false;
			return "";
		}
		else
		{			
 			String token = st.nextToken();
			if (lastIsDelim == false)
			{
				if (delim.indexOf(token) != -1)
				{
					lastIsDelim = true;
					if (returnDelims)
						return token;
					else
						return nextElement();
				}
				else
					return token;
			}
			else
			{
				if (delim.indexOf(token) != -1)
				{
					if (returnDelims)
						buffer.addLast(token);
					return "";
				}
				else
				{
					lastIsDelim = false;
					return token;
				}
			}
		}
	}

	public String nextToken()
	{
		return (String)nextElement();
	}

// This method is not implemented:
//	public String nextToken(String delim)
}