package org.gersteinlab.regulation.util;

/**
 *	<pre>
 *	This class provides some utility functions.
 *	</pre>
 *
 *	@author		Kevin Yuk-Lap Yip
 *	@version	1.0 (November 5, 2007)
 *
 *	<pre>
 *	Change History:
 *	1.0	- Initial version
 *	</pre>
 */


public class Util
{
	/*----------------------------------------------------------------------
	 Public methods
	----------------------------------------------------------------------*/

	/**
	 *	Sort an array of numbers in ascending order, and arrange the
	 *	entries in a second array accordingly.
	 *	@param		values			The values to be sorted
	 *	@param		auxValues		The auxilary values to be ordered accordingly
	 */
	public static void coSort(double[] values, Object[] auxValues)
	{
		// A merge sort algorithm
		coSort(values, auxValues, true);
	}

	/**
	 *	Sort an array of numbers, and arrange the entries in a second
	 *	array accordingly.
	 *	@param		values			The values to be sorted
	 *	@param		auxValues		The auxilary values to be ordered accordingly
	 *	@param		ascending		Whether the list is to be sorted in ascending order
	 */
	public static void coSort(double[] values, Object[] auxValues, boolean ascending)
	{
		// A merge sort algorithm
		coSort(values, auxValues, ascending, 0, values.length-1);
	}

	/**
	 *	Sort a continuous subarray of an array of numbers, and arrange
	 *	the entries in a second array accordingly.
	 *	@param		values			The values to be sorted
	 *	@param		auxValues		The auxilary values to be ordered accordingly
	 *	@param		ascending		Whether the list is to be sorted in ascending order
	 *	@param		start			The index of the first element in the subarray
	 *	@param		end			The index of the last element in the subarray
	 */
	public static void coSort(double[] values, Object[] auxValues, boolean ascending, int start, int end)
	{
		if (values.length == 0)
			return;
		if (start == end)
			return;
		else if (start == end - 1)
		{
			if (Double.isNaN(values[start]) ||
			    (ascending && values[start] > values[end]) ||
			    (!ascending && values[start] < values[end]))
			{
				double temp = values[start];
				values[start] = values[end];
				values[end] = temp;

				Object temp2 = auxValues[start];
				auxValues[start] = auxValues[end];
				auxValues[end] = temp2;
			}
		}
		else
		{
			int mid = (start + end) / 2;
			coSort(values, auxValues, ascending, start, mid);
			coSort(values, auxValues, ascending, mid+1, end);

			// Merging
			double[] tempValues = new double[end - start + 1];
			Object[] tempAuxValues = new Object[end - start + 1];
			int ptr1 = start;
			int ptr2 = mid + 1;
			for (int i=0; i<tempValues.length; i++)
			{
				if (ptr1 > mid)		// List one is already finished
				{
					tempValues[i] = values[ptr2];
					tempAuxValues[i] = auxValues[ptr2];
					ptr2 ++;
				}
				else if (ptr2 > end)	// List two is already finished
				{
					tempValues[i] = values[ptr1];
					tempAuxValues[i] = auxValues[ptr1];
					ptr1 ++;
				}
				else			// Both lists still have numbers
				{
					if (Double.isNaN(values[ptr2]) ||
					    (ascending && values[ptr1] < values[ptr2]) ||
					    (!ascending && values[ptr1] > values[ptr2]))
					{
						tempValues[i] = values[ptr1];
						tempAuxValues[i] = auxValues[ptr1];
						ptr1 ++;
					}
					else
					{
						tempValues[i] = values[ptr2];
						tempAuxValues[i] = auxValues[ptr2];
						ptr2 ++;
					}
				}
			}
			for (int i=0; i<tempValues.length; i++)
			{
				values[start+i] = tempValues[i];
				auxValues[start+i] = tempAuxValues[i];
			}
		}
	}
}